package require Thread

set N_THREADS 5
set N_TASKS   10

set T_PTR     0
set T_LIST    [list]


#
# Create single logging thread - don't have to do this, but just
# for illustration...
#
set logger [thread::create {source logger.tcl}]

#
# Create artificial list of tasks
#
for {set i 0} {$i < $N_TASKS} {incr i} {
	lappend T_LIST $i
}

#
# Create and configure set of worker threads
#
for {set i 0} {$i < $N_THREADS} {incr i} {

	set t [thread::create -joinable {source worker.tcl}]

	#
	# Define values for logger and boss thread ids in the worker
	#
	thread::send -async $t [list config logger $logger]
	thread::send -async $t [list config boss   [thread::id]]

	#
	# Tell the worker to get started
	#
	thread::send -async $t [list start]
}

set joins_left $N_THREADS
set finished   0
set joinable   [list]


#
# Handler for task requests from worker threads - if there are remaining
# tasks on the list, give out a task, otherwise tell the thread that the
# job's done
#
proc next_task {tid} {

	global T_LIST
	global T_PTR
	global N_TASKS

	if {$T_PTR < $N_TASKS} {
		set TASK [lindex $T_LIST $T_PTR]
		thread::send -async $tid [list handle task $TASK]
		incr T_PTR
	} else {
		thread::send -async $tid [list stop]
	}
}


#
# Notification from a worker that it is exiting - add the worker thread
# id to the list of threads we will join later (below) - when we have no
# more threads left to exit, we set the "finished" variable which will
# make the vwait (below) return
#
proc worker_exit tid {
	global joins_left finished joinable
	incr joins_left -1
	lappend joinable $tid
	puts "BOSS: worker_exit $tid"
	if {$joins_left == 0} {
		set finished 1
	}
}

vwait finished

puts "BOSS: done..."

#
# All done so terminate the logger - use "-async" to avoid begin told that
# the thread died while this call is in progress...
#
thread::send -async $logger terminate
